/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.annotation.editingsupport;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.conqat.lib.commons.collections.CollectionUtils;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.DialogCellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.mihalis.opal.itemSelector.DLItem;
import org.mihalis.opal.itemSelector.DualList;

/**
 * {@link CellEditor} that opens a dialog for the selection of multiple elements.
 * 
 * @author barner
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash: D8DFB89FBEC85495151411C232708CEA
 * 
 */

public class MultiSelectionCellEditor extends DialogCellEditor {

	/** Mapping: label shown in GUI -> actual value. */
	private LabelValueMapping labelValueMapping;

	/** The collection of elements selected using this {@link MultiSelectionCellEditor}. */
	private Collection<?> selectedElements = CollectionUtils.emptyList();

	/** Constructors a new MultiSelectionCellEditor */
	public MultiSelectionCellEditor(Composite parent) {
		super(parent);

	}

	/**
	 * Updates the data shown in this {@link MultiSelectionCellEditor}.
	 * 
	 * @param labelValueMapping
	 *            Mapping: label shown in GUI -> actual value.
	 * @param selectedElements
	 *            Current collection of selected elements
	 */
	public void updateData(LabelValueMapping labelValueMapping, Collection<?> selectedElements) {
		this.labelValueMapping = labelValueMapping;
		this.selectedElements = selectedElements;
	}

	/**
	 * Returns the set of selected elements. In case the user presses cancel in the
	 * {@link MultiSelectionDialog} wrapped by this {@link MultiSelectionCellEditor}, {@code null}
	 * is returned.
	 */
	public Collection<?> getSelectedElements() {
		return selectedElements;
	}

	/** {@inheritDoc} */
	@Override
	protected Object openDialogBox(Control cellEditorWindow) {

		MultiSelectionDialog selectionDialog = new MultiSelectionDialog(null);
		selectionDialog.open();

		return selectedElements;
	}

	/** Shows a dialog to that allows to select multiple values. */
	private class MultiSelectionDialog extends Dialog {

		/** Element selection dialog */
		DualList dl = null;

		/** Constructs a new {@link MultiSelectionDialog}. */
		public MultiSelectionDialog(Shell parentShell) {
			super(parentShell);
		}

		/**
		 * Creates a dummy image required to work around a layout bug in {@link DualList} if used
		 * without any image.
		 */
		private Image createDummyImage(Display display) {
			Image image = new Image(display, 1, 1);
			GC gc = new GC(image);
			gc.setBackground(display.getSystemColor(SWT.COLOR_LIST_BACKGROUND));
			gc.fillRectangle(new Rectangle(0, 0, 1, 1));
			gc.dispose();
			return image;
		}

		/** {@inheritDoc} */
		@Override
		protected Composite createDialogArea(Composite parent) {
			/* Generate the dialog and its layout */
			Composite area = (Composite)super.createDialogArea(parent);
			area.setLayout(new FillLayout());

			Set<String> labels = new HashSet<String>();
			labels.addAll(labelValueMapping.getLabels());

			Image dummyImage = createDummyImage(getShell().getDisplay());
			dl = new DualList(area, SWT.NONE);
			for(Object selectedItem : selectedElements) {
				String label = labelValueMapping.getLabelForValue(selectedItem);
				DLItem item = new DLItem(label, dummyImage);
				dl.add(item);
				dl.selectDoNotFireEvent(0);
				dl.remove(item);
				labels.remove(label);
			}

			for(String label : labels) {
				DLItem item = new DLItem(label, dummyImage);
				dl.add(item);
			}

			return area;
		}

		/** {@inheritDoc} */
		@Override
		protected void configureShell(Shell newShell) {
			super.configureShell(newShell);
			newShell.setText("Select multiple elements");
		}

		/** {@inheritDoc} */
		@Override
		protected Point getInitialSize() {
			return new Point(600, 600);
		}

		/** Returns the set of selected elements */
		private Collection<Object> getSelectedItems() {
			Collection<Object> rval = new ArrayList<Object>();
			for(DLItem item : dl.getSelection()) {
				if(item != null) {
					rval.add(labelValueMapping.getValueForLabel(item.getText()));
				}
			}
			return rval;
		}

		/** {@inheritDoc} */
		@Override
		protected void okPressed() {
			// In case the user presses ok, update the corresponding field of
			// MultiSelectionCellEditor that wraps this MultiSelectionDialog
			selectedElements = getSelectedItems();

			super.okPressed();
		}

		/** {@inheritDoc} */
		@Override
		protected void cancelPressed() {
			selectedElements = null;

			super.cancelPressed();
		}
	}
}
